/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "createArchiveThread.h"

#include "archiveHandle.h"
#include "kdar.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarCheckFileSizeEvent.h"
#include "kdarListViewItem.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"
#include "openArchiveThread.h"

#include <kcursor.h>
#include <kdebug.h>

#include <qcheckbox.h>
#include <qstring.h>
#include <qlistview.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;

extern libdar::archive *theArchive;

createArchiveThread::createArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

createArchiveThread::~createArchiveThread()
{
}

void createArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	kdarStatusBarEvent *statusEventBegin = new kdarStatusBarEvent( "Creating archive..." );	
	QApplication::postEvent( widget, statusEventBegin );
	archiveHandle archiveHndl( m_kdc->newArchiveName() );
	libdar::path fs_root( m_kdc->directoryToBackup().latin1() );
	kdDebug() << "The fs_root is " << fs_root.display().c_str() << endl;

	//Run the backup...
	//Start the creationCheckFileSizeTimer
	kdarCheckFileSizeEvent *checkFileSizeEventStart = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STARTTIMER );
	QApplication::postEvent( widget, checkFileSizeEventStart );
	
	//Display some statistics of the operation when it's finished.
	libdar::statistics st;
	
	if ( !m_kdc->differentialBackup() )
	{
		try
		{
			st=op_create(fs_root, archiveHndl.getArchiveLibdarPath(), NULL, m_kdc->libdarCreateFileMask(), m_kdc->libdarCreateDirectoryMask(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), m_kdc->pauseBetweenSlices(), m_kdc->keepPruned(), (libdar::compression) m_kdc->compressionAlgorithm(), (libdar::U_I) m_kdc->compressionLevel(), m_kdc->libdarSliceSize(), m_kdc->libdarFirstSliceSize(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->executeCommand().latin1(), libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->libdarCreateCompressionMask(), (libdar::U_I) m_kdc->minimumCompressionSizeInt(), m_kdc->ignoreDump(), m_kdc->ignoreID(), (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );
			
			kdarStatusBarEvent *statusEventEnd0 = new kdarStatusBarEvent( "Creating archive...done." );	
			QApplication::postEvent( widget, statusEventEnd0 );
		}
		catch(libdar::Egeneric &e)
		{
			libdar::user_interaction_warning(e.get_message());
		}
	}
	else
	{
		//TODO: fix a problem with the differential backup operation.. JKB040711
		//set this to the path of the reference archive, NULL if full backup.
		if (widget->openThread->running())
		{
			widget->openThread->wait();
		}
		if (theArchive != 0)
		{
			try
			{
				st=op_create(fs_root, archiveHndl.getArchiveLibdarPath(), theArchive, m_kdc->libdarCreateFileMask(), m_kdc->libdarCreateDirectoryMask(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), m_kdc->pauseBetweenSlices(), m_kdc->keepPruned(), (libdar::compression) m_kdc->compressionAlgorithm(), (libdar::U_I) m_kdc->compressionLevel(), m_kdc->libdarSliceSize(), m_kdc->libdarFirstSliceSize(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->executeCommand().latin1(), libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->libdarCreateCompressionMask(), (libdar::U_I) m_kdc->minimumCompressionSizeInt(), m_kdc->ignoreDump(), m_kdc->ignoreID(), (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );
				kdarStatusBarEvent *statusEventEnd1 = new kdarStatusBarEvent( "Creating archive...done." );	
				QApplication::postEvent( widget, statusEventEnd1 );
			}
			catch(libdar::Egeneric &e)
			{
				libdar::user_interaction_warning(e.get_message());
				libdarWorking = false;
			}
		}
		else
		{
			kdarStatusBarEvent *statusEventAborted = new kdarStatusBarEvent( "Could not open reference archive. Creation aborted." );	
			QApplication::postEvent( widget, statusEventAborted );
		}
	}
	///TODO: Report some statistics...
	
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	kdarCheckFileSizeEvent *checkFileSizeEventStop = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STOPTIMER );
	QApplication::postEvent( widget, checkFileSizeEventStop );

	libdarWorking = false;
}

void createArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
