/***************************************************************************
 *   Copyright (C) 2003 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


#include "kdar.h"
#include "kdarcreatearchivewizard.h"
#include <kfiledialog.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qwizard.h>

#include <kurlrequester.h>
#include <klineedit.h>
#include <kdebug.h>

#include <iostream>

kdarCreateArchiveWizard::kdarCreateArchiveWizard(QWidget* parent, const char* name, bool modal, WFlags fl)
        : kdarCreateWizardDlg(parent,name, modal,fl),
		  directoryDlg( new kdarDirectoryFilterDialog),
		  fileDlg( new kdarFileFilterDialog ),
		  compressionDlg( new kdarCompressionFilterDialog )
{
	rootDirectoryKURLRequester->setMode(26); //existing local directory
	rootDirectoryKURLRequester->setCaption("Select the directory you want to backup:");
	archiveNameKURLRequester->setMode(17); //local file
	archiveNameKURLRequester->setCaption("Select a name for this archive:");
	referenceArchiveKURLRequester->setMode(25); //existing local file
	referenceArchiveKURLRequester->setFilter("*.dar"); //dar archives
	referenceArchiveKURLRequester->setCaption("Select an archive to use as a reference for this backup:");
	//Disable the next buttons of the first two pages:
	//This is the archive name page: name must be entered to continue...
	setNextEnabled(page(0), false);
	//The archive root directory page: must be selected to continue...
	setNextEnabled(page(1), false);
	//Enable the finish button at the end of the wizard. 
	setFinishEnabled(page(4), true);
}

kdarCreateArchiveWizard::~kdarCreateArchiveWizard()
{}

/*$SPECIALIZATION$*/
void kdarCreateArchiveWizard::back()
{
    QWizard::back();
}

void kdarCreateArchiveWizard::next()
{
	updateDirectoryFilter();
    QWizard::next();
}

void kdarCreateArchiveWizard::changeNextButtonState(const QString& text)
{
	if (text != QString(""))
	{
		setNextEnabled(currentPage(), true);
	}
	else
	{
		setNextEnabled(currentPage(), false);
	}
}

void kdarCreateArchiveWizard::updateDirectoryFilter()
{
/*	QString theSelection = archiveNameKURLRequester->lineEdit()->text();
	QString rootDirectory = rootDirectoryKURLRequester->lineEdit()->text();
	
	if (theSelection == QString("") || rootDirectory == QString("")) return;
	
	if (theSelection[0] != '/')
	{
		KURL archiveStorageDirKURL(archiveStorageDir);
		newArchiveName = archiveStorageDirKURL.path(1) + theSelection;
	}
	else
	{
		newArchiveName = theSelection;
	}
	
	if (newArchiveName.left(rootDirectory.length()) == rootDirectory)
	{
		//the new archive name is in a subdirectory of
		//the root directory, so put the new archive directory
		//in the list of excluded paths
		KURL newArchiveKURL( newArchiveName );
		QString newArchiveDirectory = newArchiveKURL.directory(true, false);
		//Check to make sure the Directory filter list does not already contain
		//this entry:
		bool notInList = true;
		QListViewItem * curItem = directoryFiltersListView->firstChild();
		while (curItem)
		{
			if (curItem->text(0) == QString("Exclude") && curItem->text(1) == newArchiveDirectory)
			{
				notInList = false;
			}
			curItem = curItem->nextSibling();
		}
		if (notInList)
		{
			(void) new QListViewItem(directoryFiltersListView, QString("Exclude"), newArchiveDirectory);
		}
	}
*/
}

void kdarCreateArchiveWizard::slotRemoveCompressionFilter()
{
	QListViewItem * currItem = compressionFiltersListView->selectedItem();
	if (!currItem) return;
	delete currItem;
}

void kdarCreateArchiveWizard::slotRemoveAllCompressionFilter()
{
	compressionFiltersListView->clear();
}

void kdarCreateArchiveWizard::slotEditCompressionFilter()
{
}

void kdarCreateArchiveWizard::slotNewCompressionFilter()
{
/*	if (compressionDlg->exec())
	{
		QString compressionMask = compressionDlg->compressionFilterValueComboBox->currentText();
		if (compressionMask != QString(""))
		{
			if (compressionMask != QString("all of the above"))
			{
				QString type = "FIX ME!";
				// compressionDlg->compressionFilterTypeComboBox->currentText();
				(void) new QListViewItem(compressionFiltersListView, type, compressionMask);
			}
			else
			{
				QString type = "FIX ME!";
				// compressionDlg->compressionFilterTypeComboBox->currentText();
				for (int i = 0; i<(compressionDlg->compressionFilterValueComboBox->count()-1);++i)
				{
					compressionMask = compressionDlg->compressionFilterValueComboBox->text(i);
					(void) new QListViewItem(compressionFiltersListView, type, compressionMask);
				}
			}
		}
		else
		{
			kdDebug() << "Mask expression not specified." << rootDirectory.latin1() << endl;
		}
	}
*/
}

void kdarCreateArchiveWizard::slotRemoveDirectoryFilter()
{
	QListViewItem * currItem = directoryFiltersListView->selectedItem();
	if (!currItem) return;
	delete currItem;
}

void kdarCreateArchiveWizard::slotRemoveAllDirectoryFilter()
{
	directoryFiltersListView->clear();
}


void kdarCreateArchiveWizard::slotRemoveFileFilter()
{
	QListViewItem * currItem = fileFiltersListView->selectedItem();
	if (!currItem) return;
	delete currItem;
}

void kdarCreateArchiveWizard::slotRemoveAllFileFilter()
{
	fileFiltersListView->clear();
}

void kdarCreateArchiveWizard::slotEditFileFilter()
{
/*	QListViewItem * currItem = fileFiltersListView->currentItem();
	if (!currItem) return;

	QString text = currItem->text(0);
	if (text == QString("Include"))
	{
		fileDlg->fileFilterTypeComboBox->setCurrentItem(0);
	}
	else if (text == QString("Exclude"))
	{
			fileDlg->fileFilterTypeComboBox->setCurrentItem(1);
	}
	fileDlg->fileFilterValueLineEdit->setText(currItem->text(1));

	if (fileDlg->exec())
	{
		QString subdir = fileDlg->fileFilterValueLineEdit->text();
		if (subdir != rootDirectory)
		{
			//QString type = fileDlg->fileFilterTypeComboBox->currentText();
			QString type = "FIX ME!";
			currItem->setText(0, type);
			currItem->setText(1, subdir);
		}
	}
*/
}

void kdarCreateArchiveWizard::slotEditDirectoryFilter()
{
/*	QListViewItem * currItem = directoryFiltersListView->currentItem();
	if (!currItem || !currItem->isSelected()) return;
	
	int type;
	QString text = currItem->text(0);
	if (text == QString("Include"))
	{
		type = 1;
	}
	else
	{
		type = 0;
	}
	if (directoryDlg->listRadioButton->isChecked())
	{
		directoryDlg->directoryListTypeComboBox->setCurrentItem(type);
	}
	else
	{
		directoryDlg->directoryFilesystemTypeComboBox->setCurrentItem(type);
	}
	directoryDlg->directoryFilterValueLineEdit->setText(currItem->text(1));

	if (directoryDlg->exec())
	{
		if (directoryDlg->listRadioButton->isChecked())
		{
			QString subDir = directoryDlg->directoryListComboBox->currentText();
			if (subDir != QString("") && subDir != QString("All of the above") && subDir != rootDirectory)
			{
				QString filterType = directoryDlg->directoryListTypeComboBox->currentText();
				currItem->setText(0, filterType);
				currItem->setText(1, subDir);
			}		
		}
		else
		{
			QString subdir = directoryDlg->directoryFilterValueLineEdit->text();
			if (subdir != rootDirectory)
			{
				QString filterType = directoryDlg->directoryFilesystemTypeComboBox->currentText();
				currItem->setText(0, filterType);
				currItem->setText(1, subdir);
			}
		}
	}
*/
}

void kdarCreateArchiveWizard::slotNewFileFilter()
{
/*	if (fileDlg->exec())
	{
		QString filter = fileDlg->fileFilterValueLineEdit->text();
		if (filter != "")
		{
			//QString type = fileDlg->fileFilterTypeComboBox->currentText();
			QString type = "FIX ME!";
			(void) new QListViewItem(fileFiltersListView, type, filter);
		}
		else
		{
			kdDebug() << "empty file filter" << endl;
		}
	}
*/
}

void kdarCreateArchiveWizard::slotNewDirectoryFilter()
{
/*	if (directoryDlg->exec())
	{
		if (directoryDlg->listRadioButton->isChecked())
		{
			QString subDir;
			subDir = directoryDlg->directoryListComboBox->currentText();
			if (subDir != QString(""))
			{
				if (subDir != QString("All of the above"))
				{
					QString type = directoryDlg->directoryListTypeComboBox->currentText();
					(void) new QListViewItem(directoryFiltersListView, type, rootDirectory+subDir);
				}
				else
				{
					QString type = directoryDlg->directoryListTypeComboBox->currentText();
					for (int i = 0; i<(directoryDlg->directoryListComboBox->count()-1);++i)
					{
						subDir = directoryDlg->directoryListComboBox->text(i);
						(void) new QListViewItem(directoryFiltersListView, type, rootDirectory+subDir);
					}
				}
			}		
		}
		else
		{	
			QString subdir = directoryDlg->directoryFilterValueLineEdit->text();
			if (subdir != rootDirectory)
			{
				QString type = directoryDlg->directoryFilesystemTypeComboBox->currentText();
				(void) new QListViewItem(directoryFiltersListView, type, subdir);
			}
			else
			{
				kdDebug() << "the subdir does not belong to " << rootDirectory.latin1() << endl;
			}
		}
	}
*/
}

#include "kdarcreatearchivewizard.moc"
