/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@shaw.ca                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "testArchiveThread.h"

#include "kdar.h"
#include "kdarConfig.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>

#include <qstring.h>

#include <dar/libdar.hpp>
#include <dar/tools.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;

testArchiveThread::testArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

testArchiveThread::~testArchiveThread()
{
}

void testArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	kdarStatusBarEvent *statusEventBegin = new kdarStatusBarEvent( "Testing archive..." );	
	QApplication::postEvent( widget, statusEventBegin );
	libdar::et_mask selection;// = new et_mask();
	selection.add_mask(libdar::bool_mask(true));
	libdar::et_mask subtree;// = new et_mask();
	subtree.add_mask(libdar::bool_mask(true));
	
	try
	{
		libdar::statistics st=op_test(theArchive,
				selection,
				subtree,
				m_kdc->verbose());
		QString testInfo;
		QString plural("s ");
		//if (st.errored == 1) plural = " ";
		
		testInfo.append(libdar::deci(st.errored).human().c_str()+QString(" file")+plural+QString("failed CRC check.\n"));
		plural = "s ";

		if (st.ignored == libdar::infinint((libdar::U_I) 1)) plural = " ";
		testInfo.append(libdar::deci(st.ignored).human().c_str()+QString(" file")+plural+QString("ignored (excluded by filters).\n"));
		plural = "s were ";
		if (st.total() == 1) plural = " was ";
		testInfo.append(QString("A total of ")+libdar::deci(st.total()).human().c_str()+QString(" file")+plural+QString("considered.\n"));
		kdarWarningEvent *warningEvent = new kdarWarningEvent( testInfo );	
		QApplication::postEvent( widget, warningEvent );
	}
	catch(libdar::Egeneric &e)
	{
		libdar::user_interaction_warning(e.get_message());
		libdarWorking = false;
	}
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	kdarStatusBarEvent *statusEventEnd = new kdarStatusBarEvent( "Testing archive...done." );	
	QApplication::postEvent( widget, statusEventEnd );
	libdarWorking = false;
}

void testArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
